%By Ippei Fujiwara, July 25, 2006
%This code is to draw impulse responses not only against standard shocks but also expectation shock 
%a la Beaudry and Portier (2004) "An Exploration into Pigou theory of Cycles" JME(51).
% Feel free to modify and use at your own risk.

%setting (all you need to specify are just 4 parameters below)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%determine which shock is expectation shock.
expshock='z';
%period for expectation is to be realized. period must be larger than 0.
period=4;
%simulate period
simperiod=40;
%impulse responses in level difference or percentage deviation
impulse_index=2;%1 for level difference and 2 for percentage deviation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
%obtain Jacobian matrix
[info,jacobimat]=stoch_simul_es(var_list_);
close all;
clc;

numx=size(iy_,2);
nums=size(Sigma_e_);
numz=numx-nums;
numeq=size(jacobimat,1);

%Matrix difference equation must take a form like A*Z(+1)+B*Z+C*Z(-1)=0,
%namely max lead as well as max lag must be one.
if size(iy_,1)~=3
    error('too many lags or too many leads')
end

%What we would like to obtain here are alpha_0, alpha_1, alpha_2, beta_0,
%beta_1 and P in
%alpha_0*Z(+1)+alpha_1*Z+alpha_2*Z(-1)+beta_0*S(+1)+beta_1*S, and
%S=P*S(-1)
%In Dynare, jacobimat is organized as 
% A*X(-1)+B*X+C*X(+1)+D*shocks, where X=(Z,S)' where zero column is
% eliminated
ABC=jacobimat(:,1:(end-nums));
DDD=jacobimat(:,(end-nums+1):end);
shockrows=find(sum(DDD,2)~=0);
varrows=[1:numeq];
varrows=setdiff(varrows, shockrows);

zero_A=find(iy_(1,:)==0);
zero_B=find(iy_(2,:)==0);
zero_C=find(iy_(3,:)==0);

ncolumnA=numx-size(zero_A,2);
ncolumnB=numx-size(zero_B,2);
ncolumnC=numx-size(zero_C,2);

AAA=jacobimat(:,[1:ncolumnA]);
BBB=jacobimat(:,[ncolumnA+1:ncolumnA+ncolumnB]);
CCC=jacobimat(:,[ncolumnA+ncolumnB+1:ncolumnA+ncolumnB+ncolumnC]);

for i=1:size(zero_A,2);
    if i<ncolumnA
        AAA=[AAA(:,1:zero_A(i)-1) zeros(numeq,1) AAA(:,zero_A(i):end)];
    else
        AAA=[AAA zeros(numeq,1)];
    end
end
clear i;

for i=1:size(zero_B,2);
    if i<ncolumnB
        BBB=[BBB(:,1:zero_B(i)-1) zeros(numeq,1) BBB(:,zero_B(i):end)];
    else
        BBB=[BBB zeros(numeq,1)];
    end
end
clear i;

for i=1:size(zero_C,2);
    if i<ncolumnC
        CCC=[CCC(:,1:zero_C(i)-1) zeros(numeq,1) CCC(:,zero_C(i):end)];
    else
        CCC=[CCC zeros(numeq,1)];
    end
end
clear i;

AAA1=AAA([varrows],:);
AAA2=AAA([shockrows],:);
BBB1=BBB([varrows],:);
BBB2=BBB([shockrows],:);
CCC1=CCC([varrows],:);
CCC2=CCC([shockrows],:);

shockcolumns=find(sum(BBB2,1)~=0);
varcolumns=[1:numx];
varcolumns=setdiff(varcolumns, shockcolumns);

AAA11=AAA1(:,[varcolumns]);
AAA12=AAA1(:,[shockcolumns]);
AAA21=AAA2(:,[varcolumns]);
AAA22=AAA2(:,[shockcolumns]);

BBB11=BBB1(:,[varcolumns]);
BBB12=BBB1(:,[shockcolumns]);
BBB21=BBB2(:,[varcolumns]);
BBB22=BBB2(:,[shockcolumns]);

CCC11=CCC1(:,[varcolumns]);
CCC12=CCC1(:,[shockcolumns]);

%check
if sum(sum(CCC2))~=0
    error('C2 must be zero matrix')
end

if sum(sum(AAA12))~=0
    error('A12 must be zero matrix')
end

if sum(sum(AAA21))~=0
    error('A21 must be zero matrix')
end

if sum(sum(BBB21))~=0
    error('B21 must be zero matrix')
end

%reorder variables to be consistent with ghx and ghu
order_var=dr_.order_var;
nnn=1;
for i=1:size(order_var,1);
    fvar=find(varcolumns==order_var(i));
    if fvar>0
        fffvar(nnn)=fvar;
        nnn=nnn+1;
    end
end
clear nnn i;

nnn=1;
for i=1:size(order_var,1);
    fshock=find(shockcolumns==order_var(i));
    if fshock>0
        fffshock(nnn)=fshock;
        nnn=nnn+1;
    end
end
clear nnn i;

CCC11=CCC11(:,[fffvar]);
BBB11=BBB11(:,[fffvar]);
AAA11=AAA11(:,[fffvar]);
CCC12=CCC12(:,[fffshock]);
BBB12=BBB12(:,[fffshock]);

AAA22=AAA22(:,[fffshock]);
BBB22=BBB22(:,[fffshock]);

%compute alpha_0 to beta_1
alpha_0=CCC11;
alpha_1=BBB11;
alpha_2=AAA11;
beta_0=CCC12;
beta_1=BBB12;

%construnt P matrix, this may be redundant
rhos=(-1)*sum(AAA22,1)./sum(BBB22,1);
PPP=zeros(nums);
for i=1:nums;
    PPP(i,i)=rhos(i);
end
clear i;

%now compute matrices A and B in
% Z=Zss+A*Z(-1)+B*S+D1*S
% S=Sss+P*S(-1)+D2*e
%In Dynare
% X=Xss+GHX*Xstate(-1)+GHU*e

GHX=dr_.ghx;
GHU=dr_.ghu;

nstatic=dr_.nstatic;
npred=dr_.npred;
nfwrd=dr_.nfwrd;

if nstatic+npred+nfwrd~=numx
    error('number of variables is not consistent')     
end

GHX=[zeros(numx,nstatic) GHX zeros(numx,nfwrd)];

for i=1:size(varcolumns,2);
    ffvar=find(order_var==varcolumns(i));
    ffffvar(i)=ffvar;
end
clear i;
ffffvar=sort(ffffvar);

for i=1:size(shockcolumns,2);
    ffshock=find(order_var==shockcolumns(i));
    ffffshock(i)=ffshock;
end
clear i;
ffffshock=sort(ffffshock);

A=GHX([ffffvar],[ffffvar]);
C12=GHX([ffffvar],[ffffshock]);
P=GHX([ffffshock],[ffffshock]);
zeromatrix=GHX([ffffshock],[ffffvar]);
D1=GHU([ffffvar],:);
D2=GHU([ffffshock],:);

B=C12*inv(P);

%check
%P=PPP
if max(max(abs(P-PPP)))>1e-8
    error('matrices are not consistent')
end

%zeromatrix must consist of zeros
if max(max(abs(zeromatrix)))>1e-8
    error('elements of zeromatrix must be zero')
end

%C12-B*P=0
if max(max(abs(C12-B*P)))>1e-8
    error('matrices are not consistent')
end

%D1-B*D2=0
if max(max(abs(D1-B*D2)))>1e-8
    error('matrices are not consistent')
end

%alpha_0*A^2+alpha_1*A+alpha_2=0
if max(max(abs(alpha_0*A^2+alpha_1*A+alpha_2)))>1e-8
    error('matrices are not consistent')
end

%(beta_0+alpha_0*B)*P+beta_1+alpha_1*B+alpha_0*A*B=0
if max(max(abs((beta_0+alpha_0*B)*P+beta_1+alpha_1*B+alpha_0*A*B)))>1e-8
    error('matrices are not consistent')
end

%next, we compute expectation shock process
lengthexp=size(expshock,2);
lengthlgy=size(lgy_,2);

expmat=zeros(lengthexp,lengthlgy);
for i=1:lengthexp;
    findletter=find(lgy_(:,i)==expshock(i));
    nn=size(findletter,1);
    for ii=1:nn;
        expmat(i,ii)=findletter(ii);
    end        
end
clear i nn ii;

sumexpmat=sum(expmat);
flagcolumns=find(sumexpmat(1,:)~=0);
expmat=expmat(:,[flagcolumns]);

for i=1:size(expmat,2);
    findrows=find(expmat==expmat(1,i));
    if size(findrows,1)==size(expmat,1)
        shocknumber=expmat(1,i);
    end
end

findshock1=find(order_var==shocknumber);
target=find(ffffshock==findshock1);%this is the place of expectational shock in P

alphas = [alpha_0, alpha_1, alpha_2];
betas = [beta_0, beta_1];

%creat betas_new;
zerobeta=zeros(size(beta_0,1),period);
beta_0_new1=beta_0(:,[1:target]);
beta_0_new2=beta_0(:,[target+1:end]);
beta_0_new=[beta_0_new1 zerobeta beta_0_new2];

beta_1_new1=beta_1(:,[1:target]);
beta_1_new2=beta_1(:,[target+1:end]);
beta_1_new=[beta_1_new1 zerobeta beta_1_new2];

betas_new=[beta_0_new beta_1_new];

% %creat P_new matrix
P_new1=P([1:target],[1:target]);
P_new2=P([target+1:end],[target+1:end]);
P_new=zeros(size(beta_0,2)+period);
ematrix=zeros(period);
ematrix([2:end],[1:period-1])=eye(period-1);
P_new([1:target],[1:target])=P_new1;
P_new([target+1:target+1+period-1],[target+1:target+1+period-1])=ematrix;
P_new(target,target+period)=1;
P_new([target+period+1:end],[target+period+1:end])=P_new2;

[B_new] = FindandcheckB(A, P_new, alphas, betas_new);

%we need to make D2_new
D2_new1=D2([1:target],[1:target]);
D2_new2=D2([target+1:end],[target+1:end]);
D2_new=zeros(size(beta_0,2)+period);
ssmatrix=eye(period)*D2(target,target);
D2_new([1:target],[1:target])=D2_new1;
D2_new([target+1:target+1+period-1],[target+1:target+1+period-1])=ssmatrix;
D2_new([target+period+1:end],[target+period+1:end])=D2_new2;

%drawing impulse responses
%according to differencestatespace.tex (since this is not % deviation
%model),
%Z=A*Z(-1)+B*S
%S=P*S(-1)+D2*e

%varorder
order_var_title=lgy_([order_var],:);
var_title=order_var_title([ffffvar],:);
shock_title=order_var_title([ffffshock],:);
numberfig=size(A,1)+1;
numberfigures=floor(numberfig/9)+1;

%steady state values for endogenous variables
ZSS1=ys_([order_var],:);
ZSS=ZSS1([ffffvar],:);

%A case without any reverse in the future
e=zeros(size(P_new,1),simperiod);
e(target+1,1)=1;

S(:,1) = D2_new*e(:,1);
Z(:,1) = B_new*S(:,1);
for ii = 2:simperiod;
    S(:,ii)=P_new*S(:,ii-1)+D2_new*e(:,ii);
    Z(:,ii)=A*Z(:,ii-1)+B_new*S(:,ii);
end

%A case with reverse in the future
e2=zeros(size(P_new,1),simperiod);
e2(target+1,1)=1;
e2(target,1+period)=-1;

S2(:,1) = D2_new*e2(:,1);
Z2(:,1) = B_new*S2(:,1);
for ii = 2:simperiod;
    S2(:,ii)=P_new*S2(:,ii-1)+D2_new*e2(:,ii);
    Z2(:,ii)=A*Z2(:,ii-1)+B_new*S2(:,ii);
end

zeroline=zeros(1,simperiod);

%case with level difference
if impulse_index==1;
    %A case without any reverse in the future   
    figure
    subplot(3,3,1)
    hold on
    plot(S(target,:),'-');
    plot([zeroline(:)],':');
    title([shock_title(target,:)]);
    axis tight
    if size(A,1)<8
        for ii=1:size(A,1)
            subplot(3,3,ii+1)
            hold on
            plot([Z(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    else
        for ii=1:8
            subplot(3,3,ii+1)
            hold on
            plot([Z(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    end

    if numberfigures~=1
        for iii=2:numberfigures
            if iii==numberfigures
                for iiii=1:(size(A,1)-8-2*(iii-2)*9);
                    subplot(3,3,iiii)
                    hold on
                    plot([Z(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            else
                for iiii=1:9
                    subplot(3,3,iiii)
                    hold on
                    plot([Z(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            end
        end
    end
    clear i ii iii iiii;
    %A case with reverse in the future
    figure
    subplot(3,3,1)
    hold on
    plot(S2(target,:),'-');
    plot([zeroline(:)],':');
    title([shock_title(target,:)]);
    axis tight
    if size(A,1)<8
        for ii=1:size(A,1)
            subplot(3,3,ii+1)
            hold on
            plot([Z2(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['not materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    else
        for ii=1:8
            subplot(3,3,ii+1)
            hold on
            plot([Z2(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['not materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    end

    if numberfigures~=1
        for iii=2:numberfigures
            if iii==numberfigures
                for iiii=1:(size(A,1)-8-2*(iii-2)*9);
                    subplot(3,3,iiii)
                    hold on
                    plot([Z2(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['not materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            else
                for iiii=1:9
                    subplot(3,3,iiii)
                    hold on
                    plot([Z2(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['not materialized: shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            end
        end
    end
    clear i ii iii iiii;

%case with percentage deviation
else
    for zz=1:size(Z,2);
        Z(:,zz)=Z(:,zz)./ZSS;
    end    
    for zz2=1:size(Z,2);
        Z2(:,zz2)=Z2(:,zz2)./ZSS;
    end
    %A case without any reverse in the future
    figure
    subplot(3,3,1)
    hold on
    plot(S(target,:),'-');
    plot([zeroline(:)],':');
    title([shock_title(target,:)]);
    axis tight
    if size(A,1)<8
        for ii=1:size(A,1)
            subplot(3,3,ii+1)
            hold on
            plot([Z(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    else
        for ii=1:8
            subplot(3,3,ii+1)
            hold on
            plot([Z(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    end

    if numberfigures~=1
        for iii=2:numberfigures
            if iii==numberfigures
                for iiii=1:(size(A,1)-8-2*(iii-2)*9);
                    subplot(3,3,iiii)
                    hold on
                    plot([Z(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            else
                for iiii=1:9
                    subplot(3,3,iiii)
                    hold on
                    plot([Z(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            end
        end
    end
    clear i ii iii iiii;

    %A case with reverse in the future
    figure
    subplot(3,3,1)
    hold on
    plot(S2(target,:),'-');
    plot([zeroline(:)],':');
    title([shock_title(target,:)]);
    axis tight
    if size(A,1)<8
        for ii=1:size(A,1)
            subplot(3,3,ii+1)
            hold on
            plot([Z2(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['not materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    else
        for ii=1:8
            subplot(3,3,ii+1)
            hold on
            plot([Z2(ii,:)],'-');
            plot([zeroline(:)],':');
            title([var_title(ii,:)]);
            axis tight
        end
        suptitle(['not materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
    end

    if numberfigures~=1
        for iii=2:numberfigures
            if iii==numberfigures
                for iiii=1:(size(A,1)-8-2*(iii-2)*9);
                    subplot(3,3,iiii)
                    hold on
                    plot([Z2(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['not materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            else
                for iiii=1:9
                    subplot(3,3,iiii)
                    hold on
                    plot([Z2(iiii+8+(iii-2)*9,:)],'-');
                    plot([zeroline(:)],':');
                    title([var_title(iiii+8+(iii-2)*9,:)]);
                    axis tight
                end
                suptitle(['not materialized: ', 'shock is given to ' shock_title(target,:), ', period is ' num2str(period)])
            end
        end
    end
    clear i ii iii iiii;
end